from pathlib import Path

CONFIG_KEY = "qualibrate_runner"
QUALIBRATE_PATH = Path().home() / ".qualibrate"
DEFAULT_CONFIG_FILENAME = "config.toml"
DEFAULT_QUALIBRATE_CONFIG_FILENAME = "qualibrate-runner.toml"
CONFIG_PATH_ENV_NAME = "QUALIBRATE_RUNNER_CONFIG_FILE"
